@@ -3,7 +3,7 @@
  *
  * T38Modem simulator - main program
  *
- * Copyright (c) 2001-2005 Vyacheslav Frolov
+ * Copyright (c) 2001-2007 Vyacheslav Frolov
  *
  * Open H323 Project
  *
@@ -24,6 +24,21 @@
  * Contributor(s): Vyacheslav Frolov
  *
  * $Log: main.cxx,v $
+ * Revision 1.45  2007/03/01 14:03:06  vfrolov
+ * Added ability to set range of ports to use
+ *
+ * Revision 1.44  2007/01/29 12:44:41  vfrolov
+ * Added ability to put args to drivers
+ *
+ * Revision 1.43  2006/12/11 10:23:38  vfrolov
+ * Fixed typo
+ *
+ * Revision 1.42  2006/04/29 20:31:28  dominance
+ * add -v option to keep debian packaging happy ;)
+ *
+ * Revision 1.41  2005/04/28 09:12:07  vfrolov
+ * Made tidy up
+ *
  * Revision 1.40  2005/02/24 11:21:11  vfrolov
  * Added ability to put list to --route option
  *
@@ -138,11 +153,10 @@
 // mostly "stolen" from OpenAM
 
 #include <ptlib.h>
-#include <ptlib/pipechan.h>
+#include <h323pdu.h>
+#include <h323t38.h>
 
 #include "version.h"
-#include <h323pdu.h>
-#include "h323t38.h"
 #include "t38engine.h"
 #include "pmodem.h"
 #include "main.h"
@@ -169,7 +183,7 @@ T38Modem::~T38Modem()
 void T38Modem::Main()
 {
   cout << GetName()
-       << " Version " << GetVersion(TRUE)
+       << " Version " << GetVersion(TRUE) << "\n"
        << " by " << GetManufacturer()
        << " on " << GetOSClass() << ' ' << GetOSName()
        << " (" << GetOSVersion() << '-' << GetOSHardware() << ")\n\n";
@@ -185,7 +199,7 @@ BOOL T38Modem::Initialise()
 {
   PConfigArgs args(GetArguments());
 
-  args.Parse(
+  args.Parse(PseudoModemDrivers::ArgSpec() +
              "p-ptty:"
              "-route:"
              "-redundancy:"
@@ -199,9 +213,11 @@ BOOL T38Modem::Initialise()
              "g-gatekeeper:"         "n-no-gatekeeper."
              "-require-gatekeeper."  "-no-require-gatekeeper."
              "h-help."
+             "v-version."
              "i-interface:"          "-no-interface."
              "-listenport:"          "-no-listenport."
              "-connectport:"         "-no-connectport."
+             "-ports:"
 #if PMEMORY_CHECK
              "-setallocationbreakpoint:"
 #endif
@@ -247,14 +263,15 @@ BOOL T38Modem::Initialise()
         "                              Mandatory if not using GK.\n"
         "  --redundancy I[L[H]]      : Set redundancy for error recovery for\n"
         "                              (I)ndication, (L)ow speed and (H)igh\n"
-        "                              speed IFP packets.\n"
-        "                              'I', 'L' and 'H' are digits.\n"
+        "                              speed IFP packets. I, L and H are digits.\n"
         "  --old-asn                 : Use original ASN.1 sequence in T.38 (06/98)\n"
         "                              Annex A (w/o CORRIGENDUM No. 1 fix).\n"
         "  -i --interface ip         : Bind to a specific interface.\n"
         "  --no-listenport           : Disable listen for incoming calls.\n"
         "  --listenport port         : Listen on a specific port.\n"
         "  --connectport port        : Connect to a specific port.\n"
+        "  --ports T:B-M[,...]       : For (T)ype set (B)ase and (M)ax ports to use.\n"
+        "                              T is 'udp', 'rtp' or 'tcp'. B and M are numbers.\n"
         "  -g --gatekeeper host      : Specify gatekeeper host.\n"
         "  -n --no-gatekeeper        : Disable gatekeeper discovery.\n"
         "  --require-gatekeeper      : Exit if gatekeeper discovery fails.\n"
@@ -268,9 +285,10 @@ BOOL T38Modem::Initialise()
         "  -u --username str         : Set the local endpoint name to str.\n"
 #if PTRACING
         "  -t --trace                : Enable trace, use multiple times for more detail.\n"
-        "  -o --output               : File for trace output, default is stderr.\n"
+        "  -o --output file          : File for trace output, default is stderr.\n"
 #endif
         "     --save                 : Save arguments in configuration file and exit.\n"
+        "  -v --version              : Display version.\n"
         "  -h --help                 : Display this help message.\n"
         "Drivers:\n";
 
@@ -283,6 +301,11 @@ BOOL T38Modem::Initialise()
     return FALSE;
   }
 
+ 
+  if (args.HasOption('v')) {
+	return FALSE;
+  }
+ 
   if (args.HasOption("save")) {
     args.Save("save");
     cout << "Arguments were saved in configuration file\n";
@@ -507,12 +530,35 @@ void MyH323EndPoint::SetOptions(MyH323Connection &/*conn*/, OpalT38Protocol *t38
   }
 }
 
-BOOL MyH323EndPoint::Initialise(PConfigArgs & args)
+BOOL MyH323EndPoint::Initialise(const PConfigArgs &args)
 {
   DisableFastStart(!args.HasOption('F'));
   DisableH245Tunneling(args.HasOption('T'));
   DisableDetectInBandDTMF(TRUE);
 
+  if (args.HasOption("ports")) {
+    PString p = args.GetOptionString("ports");
+    PStringArray ports = p.Tokenise(",\r\n", FALSE);
+
+    for (PINDEX i = 0 ; i < ports.GetSize() ; i++) {
+      p = ports[i];
+      PStringArray ps = p.Tokenise(":-", FALSE);
+      if (ps.GetSize() == 3) {
+        if (ps[0] == "udp")
+          SetUDPPorts(ps[1].AsUnsigned(), ps[2].AsUnsigned());
+        else
+        if (ps[0] == "rtp")
+          SetRtpIpPorts(ps[1].AsUnsigned(), ps[2].AsUnsigned());
+        else
+        if (ps[0] == "tcp")
+          SetTCPPorts(ps[1].AsUnsigned(), ps[2].AsUnsigned());
+      }
+    }
+    PTRACE(1, "UDP ports: " << GetUDPPortBase() << "-" << GetUDPPortMax());
+    PTRACE(1, "RTP ports: " << GetRtpIpPortBase() << "-" << GetRtpIpPortMax());
+    PTRACE(1, "TCP ports: " << GetTCPPortBase() << "-" << GetTCPPortMax());
+  }
+
   if (args.HasOption("connectport"))
     connectPort = (WORD)args.GetOptionString("connectport").AsInteger();
   else
@@ -551,7 +597,7 @@ BOOL MyH323EndPoint::Initialise(PConfigArgs & args)
         tty = atty[0];
       }
 
-      if (!pmodem_pool->CreateModem(tty, r, PCREATE_NOTIFIER(OnMyCallback)))
+      if (!pmodem_pool->CreateModem(tty, r, args, PCREATE_NOTIFIER(OnMyCallback)))
         cout << "Can't create modem for " << tty << endl;
     }
   }
@@ -594,8 +640,7 @@ BOOL MyH323EndPoint::Initialise(PConfigArgs & args)
 
 MyH323Connection::MyH323Connection(MyH323EndPoint & _ep, unsigned callReference)
   : H323Connection(_ep, callReference), ep(_ep),
-    pmodem(NULL),
-    audioWrite(NULL), audioRead(NULL)
+    pmodem(NULL)
 {
 }
 
@@ -618,12 +663,6 @@ MyH323Connection::~MyH323Connection()
 
     ep.PMFree(pmodem);
   }
-
-  if (audioWrite != NULL)
-    delete audioWrite;
-
-  if (audioRead != NULL)
-    delete audioRead;
 }
 
 void MyH323Connection::OnEstablished()
@@ -701,10 +740,8 @@ H323Connection::AnswerCallResponse
     return AnswerCallDenied;
   }
 
-  PString old = PThread::Current()->GetThreadName();
   RenameCurrentThread(pmodem->ptyName() + "(c)");
-  PTRACE(2, "MyH323Connection::AnswerCallResponse old ThreadName=" << old);
-    
+
   PStringToString request;
   request.SetAt("command", "call");
   request.SetAt("calltoken", GetCallToken());
@@ -769,20 +806,10 @@ BOOL MyH323Connection::OpenAudioChannel(BOOL isEncoding, unsigned /* bufferSize
 
   PTRACE(2, "MyH323Connection::OpenAudioChannel " << codec);
 
-  PWaitAndSignal mutex(connMutex);
-
-  if (audioWrite == NULL) {
-    audioWrite = new AudioWrite(*this);
-  }
-
-  if (audioRead == NULL) {
-    audioRead = new AudioRead(*this, HadAnsweredCall() ? T30Tone::silence : T30Tone::cng);
-  }
-
   if (isEncoding) {
-    codec.AttachChannel(audioRead, FALSE);
+    codec.AttachChannel(new AudioRead(HadAnsweredCall() ? T30Tone::silence : T30Tone::cng), TRUE);
   } else {
-    codec.AttachChannel(audioWrite, FALSE);
+    codec.AttachChannel(new AudioWrite(), TRUE);
   }
 
   return TRUE;
@@ -790,8 +817,8 @@ BOOL MyH323Connection::OpenAudioChannel(BOOL isEncoding, unsigned /* bufferSize
 
 ///////////////////////////////////////////////////////////////
 
-AudioRead::AudioRead(MyH323Connection & _conn, T30Tone::Type type)
-  : conn(_conn), t30Tone(type), closed(FALSE)
+AudioRead::AudioRead(T30Tone::Type type)
+  : t30Tone(type), closed(FALSE)
 {
 }
 
@@ -820,8 +847,8 @@ BOOL AudioRead::Close()
 
 ///////////////////////////////////////////////////////////////
 
-AudioWrite::AudioWrite(MyH323Connection & _conn)
-  : conn(_conn), closed(FALSE)
+AudioWrite::AudioWrite()
+  : closed(FALSE)
 {
 }
 
