@@ -3,7 +3,7 @@
  *
  * T38FAX Pseudo Modem
  *
- * Copyright (c) 2001-2003 Vyacheslav Frolov
+ * Copyright (c) 2001-2007 Vyacheslav Frolov
  *
  * Open H323 Project
  *
@@ -24,6 +24,12 @@
  * Contributor(s): Equivalence Pty ltd
  *
  * $Log: main.h,v $
+ * Revision 1.17  2007/01/29 12:44:41  vfrolov
+ * Added ability to put args to drivers
+ *
+ * Revision 1.16  2005/04/28 09:12:06  vfrolov
+ * Made tidy up
+ *
  * Revision 1.15  2003/12/19 15:25:27  vfrolov
  * Removed class AudioDelay (utilized PAdaptiveDelay)
  * Renamed pmodemQ to pmodem_pool
@@ -75,7 +81,6 @@
 #define _PM_MAIN_H
 
 #include <h323.h>
-#include <lid.h>
 #include <ptclib/delaychan.h>
 #include "pmutils.h"
 #include "t30tone.h"
@@ -109,7 +114,7 @@ class MyH323EndPoint : public H323EndPoint
     virtual H323Connection * CreateConnection(unsigned callReference, void *userData);
 
     // new functions
-    BOOL Initialise(PConfigArgs & args);
+    BOOL Initialise(const PConfigArgs &args);
 
     PseudoModem * PMAlloc(const PString &number) const;
     void PMFree(PseudoModem *pmodem) const;
@@ -128,8 +133,6 @@ class MyH323EndPoint : public H323EndPoint
     PDECLARE_NOTIFIER(PObject, MyH323EndPoint, OnMyCallback);
 };
 
-class AudioRead;
-class AudioWrite;
 class OpalT38Protocol;
 
 class MyH323Connection : public H323Connection
@@ -156,8 +159,6 @@ class MyH323Connection : public H323Connection
     
     PMutex        connMutex;
     PseudoModem * pmodem;
-    AudioWrite  * audioWrite;
-    AudioRead   * audioRead;
 };
 
 ///////////////////////////////////////////////////////////////
@@ -167,12 +168,11 @@ class AudioRead : public PChannel
   PCLASSINFO(AudioRead, PChannel);
 
   public:
-    AudioRead(MyH323Connection & conn, T30Tone::Type type);
+    AudioRead(T30Tone::Type type);
     BOOL Read(void * buffer, PINDEX amount);
     BOOL Close();
 
   protected:
-    MyH323Connection & conn;
     T30Tone t30Tone;
     BOOL closed;
     PAdaptiveDelay delay;
@@ -184,12 +184,11 @@ class AudioWrite : public PChannel
   PCLASSINFO(AudioWrite, PChannel)
 
   public:
-    AudioWrite(MyH323Connection & conn);
+    AudioWrite();
     BOOL Write(const void * buf, PINDEX len);
     BOOL Close();
 
   protected:
-    MyH323Connection & conn;
     BOOL closed;
     PAdaptiveDelay delay;
     PMutex Mutex;
