@@ -3,7 +3,7 @@
  *
  * T38FAX Pseudo Modem
  *
- * Copyright (c) 2001-2004 Vyacheslav Frolov
+ * Copyright (c) 2001-2007 Vyacheslav Frolov
  *
  * Open H323 Project
  *
@@ -24,6 +24,12 @@
  * Contributor(s): Equivalence Pty ltd
  *
  * $Log: pmodem.cxx,v $
+ * Revision 1.9  2007/01/29 12:44:41  vfrolov
+ * Added ability to put args to drivers
+ *
+ * Revision 1.8  2006/05/23 14:48:54  vfrolov
+ * Fixed race condition (reported by Tamas)
+ *
  * Revision 1.7  2004/07/07 12:38:32  vfrolov
  * The code for pseudo-tty (pty) devices that communicates with fax application formed to PTY driver.
  *
@@ -68,6 +74,7 @@ class PseudoModemList : protected _PseudoModemList
     PINDEX Append(PseudoModem *modem);
     PseudoModem *Find(const PString &modemToken) const;
   protected:
+    PseudoModem *_Find(const PString &modemToken) const;
     PMutex Mutex;
 };
 
@@ -75,7 +82,7 @@ PINDEX PseudoModemList::Append(PseudoModem *modem)
 {
   PWaitAndSignal mutexWait(Mutex);
 
-  if (Find(modem->modemToken())) {
+  if (_Find(modem->modemToken())) {
     myPTRACE(1, "PseudoModemList::Append can't add " << modem->ptyName() << " to modem list");
     delete modem;
     return P_MAX_INDEX;
@@ -89,6 +96,12 @@ PINDEX PseudoModemList::Append(PseudoModem *modem)
 }
 
 PseudoModem *PseudoModemList::Find(const PString &modemToken) const
+{
+  PWaitAndSignal mutexWait(Mutex);
+  return _Find(modemToken);
+}
+
+PseudoModem *PseudoModemList::_Find(const PString &modemToken) const
 {
   PObject *object;
 
@@ -121,10 +134,11 @@ PseudoModemQ::~PseudoModemQ()
 BOOL PseudoModemQ::CreateModem(
     const PString &tty,
     const PString &route,
+    const PConfigArgs &args,
     const PNotifier &callbackEndPoint
 )
 {
-  PseudoModem *modem = PseudoModemDrivers::CreateModem(tty, route, callbackEndPoint);
+  PseudoModem *modem = PseudoModemDrivers::CreateModem(tty, route, args, callbackEndPoint);
 
   if (!modem)
       return FALSE;
