@@ -3,7 +3,7 @@
  *
  * T38FAX Pseudo Modem
  *
- * Copyright (c) 2001-2004 Vyacheslav Frolov
+ * Copyright (c) 2001-2007 Vyacheslav Frolov
  *
  * Open H323 Project
  *
@@ -24,6 +24,9 @@
  * Contributor(s): Equivalence Pty ltd
  *
  * $Log: pmodem.h,v $
+ * Revision 1.6  2007/01/29 12:44:41  vfrolov
+ * Added ability to put args to drivers
+ *
  * Revision 1.5  2004/07/07 12:38:32  vfrolov
  * The code for pseudo-tty (pty) devices that communicates with fax application formed to PTY driver.
  *
@@ -99,6 +102,7 @@ class PseudoModemQ : protected _PseudoModemQ
     BOOL CreateModem(
       const PString &tty,
       const PString &route,
+      const PConfigArgs &args,
       const PNotifier &callbackEndPoint
     );
     void Enqueue(PseudoModem *modem);
