@@ -3,7 +3,7 @@
  *
  * T38FAX Pseudo Modem
  *
- * Copyright (c) 2001-2005 Vyacheslav Frolov
+ * Copyright (c) 2001-2007 Vyacheslav Frolov
  *
  * Open H323 Project
  *
@@ -24,6 +24,29 @@
  * Contributor(s): Equivalence Pty ltd
  *
  * $Log: pmodeme.cxx,v $
+ * Revision 1.35  2007/02/22 16:00:33  vfrolov
+ * Implemented AT#HCLR command
+ *
+ * Revision 1.34  2006/12/11 11:19:48  vfrolov
+ * Fixed race condition with modem Callback
+ *
+ * Revision 1.33  2006/12/07 10:53:24  vfrolov
+ * Added OnParentStop()
+ *
+ * Revision 1.32  2006/12/01 13:35:25  vfrolov
+ * Fixed modem locking after unexpected dial while connection
+ *
+ * Revision 1.31  2005/11/22 16:39:36  vfrolov
+ * Fixed MSVC compile warning
+ *
+ * Revision 1.30  2005/03/05 15:42:39  vfrolov
+ * Added missing check for PTRACING
+ * Fixed typo in T38DLE trace
+ *
+ * Revision 1.29  2005/03/04 16:35:38  vfrolov
+ * Implemented AT#DFRMC command
+ * Redisigned class Profile
+ *
  * Revision 1.28  2005/02/16 12:14:47  vfrolov
  * Send CONNECT just before data for AT+FRM command
  *
@@ -153,64 +176,29 @@ static const char Revision[] = TOSTR(MAJOR_VERSION) "." TOSTR(MINOR_VERSION) "."
 class Profile
 {
   enum {
-    FloReg = 49,
-    CidModeReg = 50,
     MaxReg = 50,
     MaxBit = 7
   };
-  
+
   public:
     Profile();
-  
-    void Echo(BOOL val) {
-      SetBit(23, 0, val);
-    }
 
-    BOOL Echo() const {
-      BOOL val;
-      GetBit(23, 0, val);
-      return val;
-    }
+    #define DeclareRegisterBit(name, byte, bit)	\
+      void name(BOOL val) { SetBit(byte, bit, val); } \
+      BOOL name() const { BOOL val; GetBit(byte, bit, val); return val; }
 
-    void asciiResultCodes(BOOL val) {
-      SetBit(23, 6, val);
-    }
+    DeclareRegisterBit(Echo, 23, 0);
+    DeclareRegisterBit(asciiResultCodes, 23, 6);
+    DeclareRegisterBit(noResultCodes, 23, 7);
 
-    BOOL asciiResultCodes() const {
-      BOOL val;
-      GetBit(23, 6, val);
-      return val;
-    }
+    #define DeclareRegisterByte(name, byte)	\
+      void name(BYTE val) { SetReg(byte, val); } \
+      BYTE name() const { BYTE val; GetReg(byte, val); return val; }
 
-    void noResultCodes(BOOL val) {
-      SetBit(23, 7, val);
-    }
-
-    BOOL noResultCodes() const {
-      BOOL val;
-      GetBit(23, 7, val);
-      return val;
-    }
-
-    void CidMode(BYTE val) {
-      SetReg(CidModeReg, val);
-    }
-
-    BYTE CidMode() const {
-      BYTE val;
-      GetReg(CidModeReg, val);
-      return val;
-    }
-
-    void Flo(BYTE val) {
-      SetReg(FloReg, val);
-    }
-
-    BYTE Flo() const {
-      BYTE val;
-      GetReg(FloReg, val);
-      return val;
-    }
+    DeclareRegisterByte(ClearMode, 47);
+    DeclareRegisterByte(DelayFrmConnect, 48);
+    DeclareRegisterByte(Flo, 49);
+    DeclareRegisterByte(CidMode, 50);
 
     BOOL SetBit(PINDEX r, PINDEX b, BOOL val) {
       if( !ChkRB(r, b) ) return FALSE;
@@ -377,7 +365,8 @@ class ModemEngineBody : public PObject
   //@{
     BOOL Request(PStringToString &request);
     BOOL Attach(T38Engine *_t38engine);
-    void Detach(T38Engine *_t38engine);
+    void Detach(T38Engine *_t38engine) { PWaitAndSignal mutexWait(Mutex); _Detach(_t38engine); }
+    void OnParentStop();
     void HandleData(const PBYTEArray &buf, PBYTEArray &bresp);
     void CheckState(PBYTEArray &bresp);
     BOOL IsReady() const {
@@ -415,21 +404,30 @@ class ModemEngineBody : public PObject
         return FALSE;
       }
     }
-    
+
     BOOL RecvStart(int _dataType, int br) {
+      BOOL done = FALSE;
+
       PWaitAndSignal mutexWait(Mutex);
       dataType = _dataType;
       state = stRecvBegWait;
       timeout.Start(60000);
-      if( !t38engine || !t38engine->RecvWait(dataType, br, NextSeq()) ) {
+      if (!t38engine || !t38engine->RecvWait(dataType, br, NextSeq(), done)) {
         state = stCommand;
         timeout.Stop();
         return FALSE;
       }
+
+      if (done) {
+        state = stRecvBegHandle;
+        timeout.Stop();
+        parent.SignalDataReady();
+      }
+
       return TRUE;
     }
-    
-    void ClearCall() { PWaitAndSignal mutexWait(Mutex); _ClearCall(); }
+
+    void _Detach(T38Engine *_t38engine);
     void _ClearCall();
 
     int NextSeq() { return seq = ++seq % T38Engine::cbpUserDataMod; }
@@ -567,6 +565,8 @@ void ModemEngine::Main()
     WaitDataReady();
   }
 
+  body->OnParentStop();
+
   myPTRACE(1, "<-> Stopped" << GetThreadTimes(", CPU usage: "));
 }
 
@@ -594,7 +594,13 @@ ModemEngineBody::ModemEngineBody(ModemEngine &_parent, const PNotifier &_callbac
   : parent(_parent),
     t38engine(NULL),
     callbackEndPoint(_callbackEndPoint),
+#ifdef _MSC_VER
+#pragma warning(disable:4355) // warning C4355: 'this' : used in base member initializer list
+#endif
     myCallback(PCREATE_NOTIFIER(OnMyCallback)),
+#ifdef _MSC_VER
+#pragma warning(default:4355)
+#endif
     timerRing(myCallback, TRUE),
     timeout(myCallback),
     seq(0),
@@ -607,12 +613,27 @@ ModemEngineBody::ModemEngineBody(ModemEngine &_parent, const PNotifier &_callbac
 
 ModemEngineBody::~ModemEngineBody()
 {
-  ClearCall();
+  PWaitAndSignal mutexWait(Mutex);
 
-  if( t38engine ) {
-    myPTRACE(1, "ModemEngineBody::~ModemEngineBody t38engine was not Detached");
-    Detach(t38engine);
+  if (!CallToken().IsEmpty()) {
+    myPTRACE(1, "ModemEngineBody::~ModemEngineBody Call " << CallToken() << " was not cleared");
+    _ClearCall();
   }
+
+  if (t38engine) {
+    myPTRACE(1, "ModemEngineBody::~ModemEngineBody t38engine was not Detached");
+    _Detach(t38engine);
+  }
+}
+
+void ModemEngineBody::OnParentStop()
+{
+  PWaitAndSignal mutexWait(Mutex);
+
+  _ClearCall();
+
+  if (t38engine)
+    _Detach(t38engine);
 }
 
 void ModemEngineBody::_ClearCall()
@@ -646,7 +667,7 @@ BOOL ModemEngineBody::Request(PStringToString &request)
   
   if( command == "call" ) {
     PWaitAndSignal mutexWait(Mutex);
-    if (CallToken().IsEmpty()) {
+    if (callDirection == cdUndefined && CallToken().IsEmpty()) {
       CallToken(request("calltoken"));
       SrcNum(request("srcnum"));
       DstNum(request("dstnum"));
@@ -694,41 +715,73 @@ BOOL ModemEngineBody::Request(PStringToString &request)
 
 BOOL ModemEngineBody::Attach(T38Engine *_t38engine)
 {
+  if (_t38engine == NULL) {
+    myPTRACE(1, "ModemEngineBody::Attach _t38engine==NULL");
+    return FALSE;
+  }
+
   PTRACE(1, "ModemEngineBody::Attach");
+
   PWaitAndSignal mutexWait(Mutex);
 
-  if( t38engine != NULL ) {
-    myPTRACE(1, "ModemEngineBody::Attach other t38engine already Attached");
-    return FALSE;
+  for (;;) {
+    if (t38engine != NULL) {
+      myPTRACE(1, "ModemEngineBody::Attach Other t38engine already Attached");
+      return FALSE;
+    }
+
+    if (_t38engine->TryLockModemCallback()) {
+      if (!_t38engine->Attach(myCallback)) {
+        myPTRACE(1, "ModemEngineBody::Attach Can't Attach myCallback to _t38engine");
+        _t38engine->UnlockModemCallback();
+        return FALSE;
+      }
+      _t38engine->UnlockModemCallback();
+      t38engine = _t38engine;
+      break;
+    }
+
+    Mutex.Signal();
+    PThread::Sleep(20);
+    Mutex.Wait();
   }
-  t38engine = _t38engine;
-  if( !t38engine || !t38engine->Attach(myCallback) ) {
-    myPTRACE(1, "ModemEngineBody::Attach can't Attach myCallback to t38engine");
-    t38engine = NULL;
-    return FALSE;
-  }
-  
-  if( state == stReqModeAckWait ) {
+
+  if (state == stReqModeAckWait) {
     state = stReqModeAckHandle;
     timeout.Stop();
     parent.SignalDataReady();
   }
-  
+
   myPTRACE(1, "ModemEngineBody::Attach t38engine Attached");
   return TRUE;
 }
 
-void ModemEngineBody::Detach(T38Engine *_t38engine)
+void ModemEngineBody::_Detach(T38Engine *_t38engine)
 {
-  PWaitAndSignal mutexWait(Mutex);
-  if( t38engine == _t38engine ) {
-    t38engine = NULL;
-    myPTRACE(1, "ModemEngineBody::Detach t38engine Detached");
-  } else {
-    myPTRACE(1, "ModemEngineBody::Detach other t38engine was Attached");
+  if (_t38engine == NULL) {
+    myPTRACE(1, "ModemEngineBody::_Detach _t38engine==NULL");
+    return;
   }
-  if( _t38engine )
-    _t38engine->Detach(myCallback);
+
+  for (;;) {
+    if (t38engine != _t38engine) {
+      myPTRACE(1, "ModemEngineBody::_Detach " << (t38engine  ? "Other" : "No") << " t38engine was Attached");
+      return;
+    }
+
+    if (_t38engine->TryLockModemCallback()) {
+      _t38engine->Detach(myCallback);
+      _t38engine->UnlockModemCallback();
+      t38engine = NULL;
+      break;
+    }
+
+    Mutex.Signal();
+    PThread::Sleep(20);
+    Mutex.Wait();
+  }
+
+  myPTRACE(1, "ModemEngineBody::_Detach t38engine Detached");
 }
 
 void ModemEngineBody::OnMyCallback(PObject &from, INT extra)
@@ -898,12 +951,12 @@ BOOL ModemEngineBody::HandleClass1Cmd(const char **ppCmd, PString &resp, BOOL &o
                     BOOL res = T ? SendStart(dt, br, _resp) : RecvStart(dt, br);
 
                     if (_resp.GetLength()) {
-                      if (crlf)
+                      if (crlf) {
                         resp += "\r\n";
-                      else
+                        crlf = FALSE;
+                      } else {
                         resp += RC_PREF();
-
-                      crlf = FALSE;
+                      }
 
                       resp += _resp;
                     }
@@ -956,11 +1009,13 @@ void ModemEngineBody::HandleCmd(const PString & cmd, PString & resp)
       break;
   }
 
+#if PTRACING
   if (i) {
     PBYTEArray bin((const BYTE *)(const char *)cmd, i);
 
     myPTRACE(1, "--> " << PRTHEX(bin));
   }
+#endif
 
   const char *pCmd;
 
@@ -1030,11 +1085,12 @@ void ModemEngineBody::HandleCmd(const PString & cmd, PString & resp)
           break;
         case 'D':	// Dial
           ok = FALSE;
-          forceFaxMode = FALSE;
+
           {
             PString num;
             PString LocalPartyName;
             BOOL local = FALSE;
+            BOOL setForceFaxMode = FALSE;
           
             for( char ch ; (ch = *pCmd) != 0 && !err ; pCmd++ ) {
               if( isdigit(ch) ) {
@@ -1051,10 +1107,10 @@ void ModemEngineBody::HandleCmd(const PString & cmd, PString & resp)
                   case 'P':
                     break;
                   case 'F':
-                    forceFaxMode = TRUE;
+                    setForceFaxMode = TRUE;
                     break;
                   case 'V':
-                    forceFaxMode = FALSE;
+                    setForceFaxMode = FALSE;
                     break;
                   case 'L':
                     LocalPartyName = "";
@@ -1071,6 +1127,22 @@ void ModemEngineBody::HandleCmd(const PString & cmd, PString & resp)
             
             if (!err) {
               PWaitAndSignal mutexWait(Mutex);
+
+              if (!CallToken().IsEmpty()) {
+                _ClearCall();
+
+                if (crlf) {
+                  resp += "\r\n";
+                  crlf = FALSE;
+                } else {
+                  resp += RC_PREF();
+                }
+
+                resp += RC_NO_DIALTONE();
+                break;
+              }
+
+              forceFaxMode = setForceFaxMode;
               callDirection = cdOutgoing;
               timerRing.Stop();
               state = stConnectWait;
@@ -1096,6 +1168,13 @@ void ModemEngineBody::HandleCmd(const PString & cmd, PString & resp)
                 timeout.Stop();
                 state = stCommand;
                 if (response == "reject") {
+                  if (crlf) {
+                    resp += "\r\n";
+                    crlf = FALSE;
+                  } else {
+                    resp += RC_PREF();
+                  }
+
                   PString diag = request("diag");
                   if( diag == "noroute" )
                     resp += RC_BUSY();
@@ -1113,8 +1192,10 @@ void ModemEngineBody::HandleCmd(const PString & cmd, PString & resp)
           break;
         case 'H':	// On/Off-hook
           if( ParseNum(&pCmd, 0, 1, 0) >= 0 ) {	// ATH & ATH0
-            if( callDirection != cdUndefined )
-              ClearCall();
+            PWaitAndSignal mutexWait(Mutex);
+
+            if (callDirection != cdUndefined || P.ClearMode())
+              _ClearCall();
           } else {
             err = TRUE;
           }
@@ -1251,8 +1332,11 @@ void ModemEngineBody::HandleCmd(const PString & cmd, PString & resp)
           {
             int val = ParseNum(&pCmd, 0, 1, sizeof(Profiles)/sizeof(Profiles[0]) - 1);
             if( val >= 0 ) {
-              if( callDirection != cdUndefined )
-                ClearCall();
+              PWaitAndSignal mutexWait(Mutex);
+
+              if (callDirection != cdUndefined)
+                _ClearCall();
+
               P = Profiles[val];
             } else {
               err = TRUE;
@@ -1507,9 +1591,14 @@ void ModemEngineBody::HandleCmd(const PString & cmd, PString & resp)
               }
               break;
             case 'F':					// &F
-              if( callDirection != cdUndefined )
-                ClearCall();
-              P = Profiles[0];
+              {
+                PWaitAndSignal mutexWait(Mutex);
+
+                if (callDirection != cdUndefined)
+                  _ClearCall();
+
+                P = Profiles[0];
+              }
               break;
             case 'H':					// &H
               {
@@ -1557,6 +1646,67 @@ void ModemEngineBody::HandleCmd(const PString & cmd, PString & resp)
               default:
                 err = TRUE;
             }
+          } else
+          if (strncmp(pCmd, "DFRMC", 5) == 0) {         // #DFRMC
+            pCmd += 5;
+            switch( *pCmd++ ) {
+              case '=':
+                switch( *pCmd ) {
+                  case '?':
+                    pCmd++;
+                    resp += "\r\n(0-255)";
+                    crlf = TRUE;
+                    break;
+                  default:
+                    {
+                      int val = ParseNum(&pCmd);
+                      if (val >= 0) {
+                        P.DelayFrmConnect((BYTE)val);
+                      } else {
+                        err = TRUE;
+                      }
+                    }
+                }
+                break;
+              case '?':
+                resp.sprintf("\r\n%u", (unsigned)P.DelayFrmConnect());
+                crlf = TRUE;
+                break;
+              default:
+                err = TRUE;
+            }
+          } else
+          if (strncmp(pCmd, "HCLR", 4) == 0) {         // #HCLR
+            pCmd += 4;
+            switch( *pCmd++ ) {
+              case '=':
+                switch( *pCmd ) {
+                  case '?':
+                    pCmd++;
+                    resp += "\r\n(0,1)";
+                    crlf = TRUE;
+                    break;
+                  default:
+                    {
+                      int val = ParseNum(&pCmd);
+                      switch (val) {
+                        case 0:
+                        case 1:
+                          P.ClearMode((BYTE)val);
+                          break;
+                        default:
+                          err = TRUE;
+                      }
+                    }
+                }
+                break;
+              case '?':
+                resp.sprintf("\r\n%u", (unsigned)P.ClearMode());
+                crlf = TRUE;
+                break;
+              default:
+                err = TRUE;
+            }
           } else {
             err = TRUE;
           }
@@ -1941,17 +2091,6 @@ void ModemEngineBody::CheckState(PBYTEArray & bresp)
           }
           int count = t38engine->Recv(Buf, 1024);
 
-          if (!dataCount && count && dataType == T38Engine::dtRaw) {
-            // send CONNECT just before data for AT+FRM command
-
-            PString _resp = RC_PREF() + RC_CONNECT();
-
-            PBYTEArray _bresp((const BYTE *)(const char *)_resp, _resp.GetLength());
-
-            myPTRACE(1, "<-- " << PRTHEX(_bresp));
-            bresp.Concatenate(_bresp);
-          }
-
           switch (count) {
             case -1:
               {
@@ -1967,18 +2106,41 @@ void ModemEngineBody::CheckState(PBYTEArray & bresp)
                 dleData.SetDiag(diag).PutEof();
               }
               t38engine->RecvStop();
-              if (dataCount == 0 && dataType == T38Engine::dtHdlc)
+              if (dataCount == 0)
                 dleData.GetDleData(Buf, 1024);	// discard ...<DLE><ETX>
               break;
             case 0:
               break;
             default:
-              dataCount += count;
               dleData.PutData(Buf, count);
-              if (dataType == T38Engine::dtHdlc)
-                fcs.build(Buf, count);
+              switch (dataType) {
+                case T38Engine::dtHdlc:
+                  fcs.build(Buf, count);
+                  break;
+                case T38Engine::dtRaw:
+                  if (!dataCount) {
+                    int dms = P.DelayFrmConnect();
+
+                    if (dms) {
+                      Mutex.Signal();
+                      PThread::Sleep(dms * 10);
+                      Mutex.Wait();
+                    }
+
+                    // send CONNECT just before data for AT+FRM command
+
+                    PString _resp = RC_PREF() + RC_CONNECT();
+
+                    PBYTEArray _bresp((const BYTE *)(const char *)_resp, _resp.GetLength());
+
+                    myPTRACE(1, "<-- " << PRTHEX(_bresp));
+                    bresp.Concatenate(_bresp);
+                  }
+              }
+              dataCount += count;
           }
-          if( count <= 0 ) break;
+          if (count <= 0)
+            break;
         }
 
         for(;;) {
@@ -2009,9 +2171,9 @@ void ModemEngineBody::CheckState(PBYTEArray & bresp)
 #if PTRACING
               if (myCanTrace(2)) {
                  if (count <= 16) {
-                   PTRACE(2, "T38DLE--> " << PRTHEX(PBYTEArray(Buf, count)));
+                   PTRACE(2, "<-- T38DLE " << PRTHEX(PBYTEArray(Buf, count)));
                  } else {
-                   PTRACE(2, "T38DLE--> " << count << " bytes");
+                   PTRACE(2, "<-- T38DLE " << count << " bytes");
                  }
               }
 #endif
