@@ -3,7 +3,7 @@
  *
  * T38FAX Pseudo Modem
  *
- * Copyright (c) 2001-2005 Vyacheslav Frolov
+ * Copyright (c) 2001-2006 Vyacheslav Frolov
  *
  * Open H323 Project
  *
@@ -24,6 +24,26 @@
  * Contributor(s): Equivalence Pty ltd
  *
  * $Log: t38engine.cxx,v $
+ * Revision 1.40  2006/12/11 11:19:48  vfrolov
+ * Fixed race condition with modem Callback
+ *
+ * Revision 1.39  2006/12/07 10:50:39  vfrolov
+ * Fixed possible dead lock
+ *
+ * Revision 1.38  2006/10/20 10:04:00  vfrolov
+ * Added code for ignoring repeated indicators
+ * Added code for sending repeated indicators (disabled by default)
+ *
+ * Revision 1.37  2006/07/05 04:37:17  csoutheren
+ * Applied 1488904 - SetPromiscuous(AcceptFromLastReceivedOnly) for T.38
+ * Thanks to Vyacheslav Frolov
+ *
+ * Revision 1.36  2005/07/21 06:49:02  vfrolov
+ * Added missing CompleteEncoding()
+ *
+ * Revision 1.35  2005/07/18 11:39:47  vfrolov
+ * Changed for OPAL
+ *
  * Revision 1.34  2005/03/03 16:09:06  vfrolov
  * Fixed memory leak
  *
@@ -140,9 +160,16 @@
  */
 
 #include <ptlib.h>
+
+#ifdef USE_OPAL
+  #include <opal/transports.h>
+  #include <asn/t38.h>
+#else
+  #include <transports.h>
+  #include <t38.h>
+#endif
+
 #include "t38engine.h"
-#include <transports.h>
-#include <t38.h>
 
 #define new PNEW
 
@@ -453,10 +480,9 @@ T38Engine::~T38Engine()
 BOOL T38Engine::Attach(const PNotifier &callback)
 {
   PTRACE(1, name << " T38Engine::Attach");
-  PWaitAndSignal mutexWaitModem(MutexModem);
-  PWaitAndSignal mutexWaitModemCallback(MutexModemCallback);
   PWaitAndSignal mutexWait(Mutex);
-  if( !modemCallback.IsNULL() ) {
+
+  if (!modemCallback.IsNULL()) {
     myPTRACE(1, name << " T38Engine::Attach !modemCallback.IsNULL()");
     return FALSE;
   }
@@ -468,19 +494,37 @@ BOOL T38Engine::Attach(const PNotifier &callback)
 void T38Engine::Detach(const PNotifier &callback)
 {
   PTRACE(1, name << " T38Engine::Detach");
-  PWaitAndSignal mutexWaitModem(MutexModem);
-  PWaitAndSignal mutexWaitModemCallback(MutexModemCallback);
   PWaitAndSignal mutexWait(Mutex);
-  if( modemCallback == callback ) {
+
+  if (modemCallback == callback) {
     modemCallback = NULL;
     _ResetModemState();
     T38Mode = FALSE;
-    myPTRACE(1, name << " T38Engine::Detach T38Mode=FALSE");
+    myPTRACE(1, name << " T38Engine::Detach Detached");
     SignalOutDataReady();
   } else {
-    myPTRACE(1, name << " T38Engine::Detach modemCallback != callback");
+    myPTRACE(1, name << " T38Engine::Detach "
+      << (modemCallback.IsNULL() ? "Already Detached" : "modemCallback != callback"));
   }
 }
+
+BOOL T38Engine::TryLockModemCallback()
+{
+  MutexModem.Wait();
+
+  if (!MutexModemCallback.Wait(0)) {
+    MutexModem.Signal();
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
+void T38Engine::UnlockModemCallback()
+{
+  MutexModemCallback.Signal();
+  MutexModem.Signal();
+}
 ///////////////////////////////////////////////////////////////
 void T38Engine::SetRedundancy(int indication, int low_speed, int high_speed) {
   if (indication >= 0)
@@ -522,10 +566,32 @@ BOOL T38Engine::Originate()
   T38_UDPTLPacket udptl;
   udptl.m_error_recovery.SetTag(T38_UDPTLPacket_error_recovery::e_secondary_ifp_packets);
 
+#if REPEAT_INDICATOR_SENDING
+  T38_IFPPacket lastifp;
+#endif
+
   for (;;) {
     T38_IFPPacket ifp;
+    int res;
 
-    int res = PreparePacket(ifp, maxRedundancy > 0);
+#ifdef REPEAT_INDICATOR_SENDING
+    if (seq >= 0 && lastifp.m_type_of_msg.GetTag() == T38_Type_of_msg::e_t30_indicator)
+      res = PreparePacket(ifp, TRUE); // do not wait next ifp
+    else
+#endif
+
+    res = PreparePacket(ifp, maxRedundancy > 0);
+
+#ifdef REPEAT_INDICATOR_SENDING
+    if (res > 0)
+      lastifp = ifp;
+    else
+    if (res < 0 && lastifp.m_type_of_msg.GetTag() == T38_Type_of_msg::e_t30_indicator) {
+      // send indicator again
+      ifp = lastifp;
+      res = 1;
+    }
+#endif
 
     if (res > 0) {
       T38_UDPTLPacket_error_recovery &recovery = udptl.m_error_recovery;
@@ -608,6 +674,7 @@ BOOL T38Engine::Originate()
 
     PPER_Stream rawData;
     udptl.Encode(rawData);
+    rawData.CompleteEncoding();
 
 #if PTRACING
     if (res > 0) {
@@ -659,9 +726,9 @@ BOOL T38Engine::Answer()
   RenameCurrentThread(name + "(rx)");
   PTRACE(2, "T38\tAnswer, transport=" << *transport);
 
-  /* HACK HACK HACK -- need to figure out how to get the remote address
-   * properly here */
-  transport->SetPromiscuous(H323Transport::AcceptFromAnyAutoSet);
+  // We can't get negotiated sender's address and port,
+  // so accept first packet from any address and port
+  transport->SetPromiscuous(transport->AcceptFromAny);
 
   int consecutiveBadPackets = 0;
   long expectedSequenceNumber = 0;
@@ -676,18 +743,18 @@ BOOL T38Engine::Answer()
       break;
     }
 
-    /* when we get the first packet, set the RemoteAddress and then turn off
-     * promiscuous listening */
-    if (expectedSequenceNumber == 0) {
-      PTRACE(3, "T38\tReceived first packet, remote=" << transport->GetRemoteAddress());
-      transport->SetPromiscuous(H323Transport::AcceptFromRemoteOnly);
-    }
-
     T38_UDPTLPacket udptl;
 
-    if (udptl.Decode(rawData))
+    if (udptl.Decode(rawData)) {
       consecutiveBadPackets = 0;
-    else {
+
+      // When we get the first packet, we know sender's address and port,
+      // so accept next packets from sender's address and port only
+      if (expectedSequenceNumber == 0) {
+        PTRACE(3, "T38\tReceived first packet, remote=" << transport->GetLastReceivedAddress());
+        transport->SetPromiscuous(transport->AcceptFromLastReceivedOnly);
+      }
+    } else {
       consecutiveBadPackets++;
       PTRACE(2, "T38\tRaw data decode failure:\n  "
              << setprecision(2) << rawData << "\n  UDPTL = "
@@ -787,18 +854,19 @@ done:
 void T38Engine::ModemCallbackWithUnlock(INT extra)
 {
   Mutex.Signal();
-  {
-    PWaitAndSignal mutexWaitModemCallback(MutexModemCallback);
-    if (!modemCallback.IsNULL())
-      modemCallback(*this, extra);
-  }
+  MutexModemCallback.Wait();
+
+  if (!modemCallback.IsNULL())
+    modemCallback(*this, extra);
+
+  MutexModemCallback.Signal();
   Mutex.Wait();
 }
 
-void T38Engine::CleanUpOnTermination()
+void T38Engine::CleanUpOnTerminationOrClose()
 {
-  myPTRACE(1, name << " T38Engine::CleanUpOnTermination");
-  OpalT38Protocol::CleanUpOnTermination();
+  myPTRACE(1, name << " T38Engine::CleanUpOnTerminationOrClose");
+  OpalT38Protocol::CleanUpOnTerminationOrClose();
 
   PWaitAndSignal mutexWait(Mutex);
   T38Mode = FALSE;
@@ -942,7 +1010,7 @@ BOOL T38Engine::SendStop(BOOL moreFrames, int _callbackParam)
   return TRUE;
 }
 ///////////////////////////////////////////////////////////////
-BOOL T38Engine::RecvWait(int _dataType, int param, int _callbackParam)
+BOOL T38Engine::RecvWait(int _dataType, int param, int _callbackParam, BOOL &done)
 {
   PWaitAndSignal mutexWaitModem(MutexModem);
   if (!IsT38Mode())
@@ -984,7 +1052,7 @@ BOOL T38Engine::RecvWait(int _dataType, int param, int _callbackParam)
         modStreamIn->ModPars.dataTypeT38 =
             (modStreamIn->ModPars.msgType == T38D(e_v21) || t30.hdlcOnly()) ? dtHdlc : dtRaw;
         stateModem = stmInReadyData;
-        ModemCallbackWithUnlock(_callbackParam);
+        done = TRUE;
         return TRUE;
       }
     }
@@ -1014,7 +1082,7 @@ BOOL T38Engine::RecvWait(int _dataType, int param, int _callbackParam)
       modStreamIn->PutEof(diagDiffSig);
     }
     stateModem = stmInReadyData;
-    ModemCallbackWithUnlock(_callbackParam);
+    done = TRUE;
     return TRUE;
   }
 
@@ -1500,6 +1568,15 @@ BOOL T38Engine::HandlePacket(const T38_IFPPacket & ifp)
 
   switch( ifp.m_type_of_msg.GetTag() ) {
     case T38_Type_of_msg::e_t30_indicator:
+      if ((modStreamIn != NULL) && (modStreamIn->lastBuf != NULL &&
+            modStreamIn->ModPars.ind == (T38_Type_of_msg_t30_indicator)ifp.m_type_of_msg) ||
+          (modStreamInSaved != NULL) && (modStreamInSaved->lastBuf != NULL &&
+            modStreamInSaved->ModPars.ind == (T38_Type_of_msg_t30_indicator)ifp.m_type_of_msg))
+      {
+        myPTRACE(3, "T38Engine::HandlePacket ignored repeated indicator " << ifp.m_type_of_msg);
+        break;
+      }
+
       if( modStreamIn != NULL && modStreamIn->PutEof(diagOutOfOrder) )
         myPTRACE(1, "T38Engine::HandlePacket indicator && modStreamIn->lastBuf != NULL");
 
