@@ -3,7 +3,7 @@
  *
  * T38FAX Pseudo Modem
  *
- * Copyright (c) 2001-2004 Vyacheslav Frolov
+ * Copyright (c) 2001-2006 Vyacheslav Frolov
  *
  * Open H323 Project
  *
@@ -24,6 +24,20 @@
  * Contributor(s): Equivalence Pty ltd
  *
  * $Log: t38engine.h,v $
+ * Revision 1.22  2006/12/22 12:51:00  vfrolov
+ * Fixed class of MutexModemCallback
+ * Added volatile to isCarrierIn
+ * Removed stateIn
+ *
+ * Revision 1.21  2006/12/11 11:19:48  vfrolov
+ * Fixed race condition with modem Callback
+ *
+ * Revision 1.20  2005/07/20 12:42:36  vfrolov
+ * Done less strict comparison in MODPARS::IsEqual()
+ *
+ * Revision 1.19  2005/07/18 11:39:48  vfrolov
+ * Changed for OPAL
+ *
  * Revision 1.18  2004/06/18 15:06:29  vfrolov
  * Fixed race condition by adding mutex for modemCallback
  *
@@ -86,8 +100,13 @@
 #ifndef _T38ENGINE_H
 #define _T38ENGINE_H
 
+#ifdef USE_OPAL
+  #include <t38/t38proto.h>
+#else
+  #include <t38proto.h>
+#endif
+
 #include "pmutils.h"
-#include <t38proto.h>
 #include "hdlc.h"
 #include "t30.h"
 
@@ -104,7 +123,7 @@ class MODPARS
     );
 
     BOOL IsModValid() const { return val >= 0; }
-    BOOL IsEqual(const MODPARS &mp) const { return val == mp.val; }
+    BOOL IsEqual(const MODPARS &mp) const { return msgType == mp.msgType; }
 
     int dataType;
     int dataTypeT38;
@@ -156,7 +175,6 @@ class T38Engine : public OpalT38Protocol
   
   /**@name Operations */
   //@{
-    void CleanUpOnTermination();
     void SetRedundancy(
       int indication,
       int low_speed,
@@ -175,6 +193,9 @@ class T38Engine : public OpalT38Protocol
   //@{
     BOOL Attach(const PNotifier &callback);
     void Detach(const PNotifier &callback);
+    BOOL TryLockModemCallback();
+    void UnlockModemCallback();
+
     void ResetModemState();
     BOOL isOutBufFull() const;
 
@@ -183,7 +204,7 @@ class T38Engine : public OpalT38Protocol
     int Send(const void *pBuf, PINDEX count);
     BOOL SendStop(BOOL moreFrames, int _callbackParam);
 
-    BOOL RecvWait(int _dataType, int param, int _callbackParam);
+    BOOL RecvWait(int _dataType, int param, int _callbackParam, BOOL &done);
     BOOL RecvStart(int _callbackParam);
     int Recv(void *pBuf, PINDEX count);
     int RecvDiag();
@@ -194,6 +215,13 @@ class T38Engine : public OpalT38Protocol
   
     void EncodeIFPPacket(PASN_OctetString &ifp_packet, const T38_IFPPacket &T38_ifp) const;
 
+#ifdef USE_OPAL
+    #define CleanUpOnTerminationOrClose Close
+#else
+    #define CleanUpOnTerminationOrClose CleanUpOnTermination
+#endif
+
+    void CleanUpOnTerminationOrClose();
     BOOL Originate();
     BOOL Answer();
 
@@ -252,9 +280,8 @@ class T38Engine : public OpalT38Protocol
     HDLC hdlcOut;
     PSyncPoint outDataReadySyncPoint;
 
-    int stateIn;
     int callbackParamIn;
-    int isCarrierIn;
+    volatile int isCarrierIn;
 #if PTRACING
     PTime timeBeginIn;
     PINDEX countIn;
@@ -273,7 +300,7 @@ class T38Engine : public OpalT38Protocol
     PMutex MutexOut;
     PMutex MutexIn;
     PMutex MutexModem;
-    PMutex MutexModemCallback;
+    PTimedMutex MutexModemCallback;
 
     const PString name;
 };
